<?php

namespace jrosset\EnvReader;

use Exception;
use jrosset\EnvReader\XmlStructure\IXmlStructure;

/**
 * A XML file configuration class
 */
abstract class XmlConfig extends GenericConfig implements IExternalConfigFile {
    use TMultiLevelProperties;

    /**
     * @inheritDoc
     */
    protected function readConfig (): void {
        //region Contrôle fichier existant
        $configFile = $this->getConfigFilePath();
        if (!file_exists($configFile)) {
            throw new Exception('Unable to find configuration file "' . $configFile . '"');
        }
        //endregion
        //region Chargement fichier et traitement erreurs
        if (($xml = simplexml_load_file($configFile)) === false) {
            $errors = [];
            foreach (libxml_get_errors() as $error) {
                $errorStr = '';
                switch ($error->level) {
                    case LIBXML_ERR_WARNING:
                        $errorStr .= 'Warning';
                    break;

                    case LIBXML_ERR_ERROR:
                        $errorStr .= 'Error';
                    break;

                    case LIBXML_ERR_FATAL:
                        $errorStr .= 'Fatal error';
                    break;
                }
                $errorStr .= ' #' . $error->code . ': ' . $error->message . ' (line: ' . $error->line . ', column: ' . $error->column . ')';
                $errors[] = $errorStr;
            }

            throw new Exception('Unable to read configuration file "' . $configFile . '": ' . PHP_EOL . implode(PHP_EOL, $errors));
        }
        //endregion

        //region Parsage XML
        $this->properties = $this->getConfigStructure()->parseXml($xml);
        //endregion
    }
    /**
     * Obtient la structure de la configuration
     *
     * @return IXmlStructure La structure de la configuration
     */
    protected abstract function getConfigStructure (): IXmlStructure;
}