<?php

namespace jrosset\EnvReader\XmlStructure;

use jrosset\Collections\InsensitiveCaseKeyCollection;
use SimpleXMLElement;

/**
 * Structure about children list
 */
class ListXmlStructure implements IXmlStructure {
    /**
     * @var string The child node's name
     */
    private string $childNodeName;
    /**
     * @var IXmlStructure The child node's structure
     */
    private IXmlStructure $childNodeStructure;

    /**
     * Initialize
     *
     * @param string        $childNodeName      The child node's name
     * @param IXmlStructure $childNodeStructure The child node's structure
     */
    public function __construct (string $childNodeName, IXmlStructure $childNodeStructure) {
        $this->childNodeName = $childNodeName;
        $this->childNodeStructure = $childNodeStructure;
    }

    /**
     * @inheritDoc
     */
    public function parseXml (SimpleXMLElement $xmlNode): InsensitiveCaseKeyCollection {
        $properties = new InsensitiveCaseKeyCollection();
        if (!isset($xmlNode->{$this->childNodeName})) {
            return $properties;
        }

        foreach ($xmlNode->{$this->childNodeName} as $childNode) {
            $childProperties = $this->childNodeStructure->parseXml($childNode);
            $properties->add($childProperties);
        }
        return $properties;
    }
}